#ifndef _HTTP_RESPONSE_H_
#define _HTTP_RESPONSE_H_

/*
   *
   * http_response.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <sys/sendfile.h>
#include <sys/stat.h>
#include <sys/types.h>


#define USR_LOCAL_GIF	"/usr/local/lib/bcount/"
#define PICS_GIF	"pics/"
#define	DOC_TYPE	"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"


void http_bad_request_response(char *request)
{
	snprintf (body_buffer, sizeof(body_buffer), http_bad_request_body, 
		    DOC_TYPE, request, SNAME, SVERSION, get_local_host_name(),
		    server_port);

	body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_bad_request_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


void http_not_found_response(const char* url)
{
	snprintf (body_buffer, sizeof(body_buffer), http_not_found_body, 
		    DOC_TYPE, url, SNAME, SVERSION, get_local_host_name(),
		    server_port);

	body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_not_found_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


void http_bad_method_response(char *method)
{
	snprintf (body_buffer, sizeof(body_buffer), http_bad_method_body, 
		    DOC_TYPE, method, SNAME, SVERSION, get_local_host_name(),
		    server_port);

	body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_bad_method_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


void http_interface_not_found_response(char *interface)
{
	snprintf (body_buffer, sizeof(body_buffer), 
		    http_interface_not_found_body, DOC_TYPE, interface,
		    SNAME, SVERSION, get_local_host_name(), server_port);

	body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), 
		    http_interface_not_found_header, HTTP_PROTOCOL_1_0, 
		    SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


/* File descriptor `write_stream' must be opened (open() function!) 
    with write rights when running this function! */
int write_gif_file_to_stream(char *file_name, int write_stream)
{
	int read_stream;
	char full_path_to_file[50];
	struct stat *stat_buf = (struct stat *) xmalloc (sizeof(struct stat));
	off_t offset = 0;

	strcpy (full_path_to_file, USR_LOCAL_GIF);
	strcat (full_path_to_file, file_name);
	
	if ((access (full_path_to_file, R_OK)) != 0)
	    {
		strcpy (full_path_to_file, PICS_GIF);
		strcat (full_path_to_file, file_name);

		if ((access (full_path_to_file, R_OK)) != 0)
		    {
			printf ("-> Search %s in %s and %s Fail\n",
				file_name, PICS_GIF, USR_LOCAL_GIF);
			printf ("-> Error read %s, can't send this "
				"to client :-(\n", file_name);

			return(EXIT_FALSE);
		    }

	    }

	read_stream = open (full_path_to_file, O_RDONLY);
	fstat (read_stream, stat_buf);

	snprintf (header_buffer, sizeof(header_buffer), http_ok_gif_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, stat_buf->st_size);
	header_buffer[strlen (header_buffer)] = '\0';
	write(write_stream, header_buffer, strlen (header_buffer));

	sendfile (write_stream, read_stream, &offset, stat_buf->st_size);

	free(stat_buf);
	close(read_stream);

	return(EXIT_SUCCESS);
}	    


void http_server_info_response(void)
{
	snprintf (body_buffer, sizeof(body_buffer),
                 http_server_info_body, DOC_TYPE, SVERSION, i->name, 
		 page_address, author_name, email_address, email_address, 
		 page_address, page_address, get_local_host_name(),
		 get_local_host_info(HOST_ARCHITECTURE),
		 get_cpu_clock_speed(), get_rss_memory(getpid()),
		 (unsigned int) getpid(), get_interface_list());
	
	free(u_name);

        body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_ok_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


void http_about_response(void)
{
	snprintf (body_buffer, sizeof(body_buffer),
                 http_about_body, DOC_TYPE, SVERSION, i->name, page_address);

        body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_ok_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


void http_interface_info_response(char *interface)
{
	snprintf (body_buffer, sizeof(body_buffer),
                 http_interface_info_body, DOC_TYPE, SVERSION, 
		 interface, page_address, interface, get_local_host_name(),
		 i->description,
		 i->eth.driver,
		 i->eth.bus_info,
		 i->ip_addr,
		 i->ip_daddr,
		 i->netmask,
		 i->broadcast, 
		 i->hw_addr, 
		 i->ip_addr_class, 
		 i->ip_daddr_class, 
		 i->metric, 
		 i->mtu, 
		 i->irq,
		 i->dma,
		 i->nflags);

        body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_ok_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}	    


void http_interface_data_response(char *interface)
{
	if((i->speed.kbs.speed_current_kbytes > 0) && 
	    (i->speed.kbs.speed_current_kbytes < 1))
	    {
		snprintf (body_buffer, sizeof(body_buffer), 
			    http_interface_data_body, DOC_TYPE, xrefresh, 
			    SVERSION, interface, page_address, interface, 
			    get_local_host_name(),
			    i->ip_addr,
			    i->stats.tx_mbytes, MBYTES, 
			    i->stats.rx_mbytes, MBYTES, 
			    i->stats.tx_packets, 
			    i->stats.rx_packets,
			    i->stats.tx_errs, 
			    i->stats.rx_errs, 
			    (unsigned int)(i->speed.kbs.speed_current_kbytes * 1024), 
			    BYTES_PER_SEC,
			    i->speed.kbs.speed_kbytes_top, 
			    KBYTES_PER_SEC,
			    (unsigned int)(i->speed.kbs.speed_current_tx_kbytes * 1024), 
			    BYTES_PER_SEC,
			    (unsigned int)(i->speed.kbs.speed_current_rx_kbytes * 1024), 
			    BYTES_PER_SEC,
			    i->speed.pss.speed_current_packets,
			    i->speed.pss.speed_packets_top,
			    i->speed.pss.speed_current_tx_packets,
			    i->speed.pss.speed_current_rx_packets,
			    get_localdate(), get_localtime(), xrefresh);
	    }
	else
	    {
		snprintf (body_buffer, sizeof(body_buffer), 
			    http_interface_kb_data_body, DOC_TYPE, xrefresh, 
			    SVERSION, interface, page_address, interface, 
			    get_local_host_name(),
			    i->ip_addr,
			    i->stats.tx_mbytes, MBYTES, 
			    i->stats.rx_mbytes, MBYTES, 
			    i->stats.tx_packets, 
			    i->stats.rx_packets,
			    i->stats.tx_errs, 
			    i->stats.rx_errs, 
			    i->speed.kbs.speed_current_kbytes, 
			    KBYTES_PER_SEC,
			    i->speed.kbs.speed_kbytes_top, 
			    KBYTES_PER_SEC,
			    i->speed.kbs.speed_current_tx_kbytes, 
			    KBYTES_PER_SEC,
			    i->speed.kbs.speed_current_rx_kbytes, 
			    KBYTES_PER_SEC,
			    i->speed.pss.speed_current_packets,
			    i->speed.pss.speed_packets_top,
			    i->speed.pss.speed_current_tx_packets,
			    i->speed.pss.speed_current_rx_packets,
			    get_localdate(), get_localtime(), xrefresh);
	    }
	    
        body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), http_ok_header, 
		    HTTP_PROTOCOL_1_0, SNAME, SVERSION, strlen (body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';
}


/* NIN */

#endif /* _HTTP_RESPONSE_H_ */

