#ifndef _BCLIENT_H_
#define _BCLIENT_H_

/*
   *
   * bclient.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <sys/types.h>
#include <arpa/inet.h>
#include <curses.h>
#include <getopt.h>
#include <netdb.h>
#include <sys/time.h>
#include "constans.h"
#include "protocol.h"
#include "includes.h"
#include "bugy_function.h"
#include "colours.h"
#include "colours_curses.h"
#include "data_type.h"
#include "programs_version.h"
#include "protocol_version.h"
#include "xmalloc.h"
#include "windows_utils.h"
#include "host_info.h"
#include "server_port.h"
#include "host_name.h"
#include "socket_stream.h"
#include "print_info.h"
#include "terminal_setting.h"


/* Client name and version */
#define CNAME		"Bytes Counter Client"

/* Def. max message which client can read */
#define MESSAGE_MAX	62


void interface_error(void) __attribute__ ((noreturn));
void request_error(void) __attribute__ ((noreturn));
void method_error(void) __attribute__ ((noreturn));
void protocol_error(void) __attribute__ ((noreturn));
void server_name_error(void) __attribute__ ((noreturn));
void port_error(void) __attribute__ ((noreturn));
void message_error(void) __attribute__ ((noreturn));
void print_client_usage(void) __attribute__ ((noreturn));

void init_ncurses_screen(void);
void draw_ncurses_screen(void);
void refresh_server_data_window(void);
int client_init(void);
int connect_to_server(void);
int send_request_to_server(char *method, char *interface, char *protocol,
			    char *data_type, char *message);
int read_welcome_from_server(void);
int get_header_and_body_from_server(void);


WINDOW *main_window;
WINDOW *server_data_window;
WINDOW *info_window;
WINDOW *connect_window;
WINDOW *message_window;
WINDOW *version_window;

int server_port = SPORT;
int content_length = 0;
char message[101] = ANULL;
char server_name[50] = ANULL;
char server_interface[50];
char server_interface_ip[80];

int csd = -1, server_y_max, server_x_max;
char server_version[15];
char bcnt_server_protocol[10];
char server_answer[20];
char server_message[63];
char server_packets_rx[20], server_packets_tx[20];
char server_errs_rx[20], server_errs_tx[20];
char server_bytes_rx[20], server_bytes_tx[20];
ssize_t bytes_read = 0;
char transmit_buffer[4024];
char received_buffer[4024];
struct sockaddr_in address;



struct option longopts[] = {
	{"server",	    1, 0, 's'},
	{"interface",       1, 0, 'i'},
	{"port",            1, 0, 'p'},
	{"data",            1, 0, 'd'},
	{"message",         1, 0, 'm'},
	{"version",         0, 0, 'v'},
	{"license",         0, 0, 'l'},
	{"help",            0, 0, 'h'},
	{ 0, 0, 0, 0 }
};


void print_client_version(void)
{
	printf ("BYTES COUNTER CLIENT v"_STRING_WHITE_, CVERSION);
}


void print_welcome(void)
{
	printf ("-== ");
	print_client_version();
	printf (" on an %s running... ==-\n", 
		get_local_host_info(HOST_ARCHITECTURE));

	free(u_name);
}


void interface_error(void)
{
	close_ncurses_screen();
	printf ("This interface on server not found, exiting...\n");

	exit(EXIT_FALSE);
}


void request_error(void)
{
	close_ncurses_screen();
	printf ("Your request which you sent to server is bad.\n"
		"Server can't understand this!, exiting...\n");

	exit(EXIT_FALSE);
}


void method_error(void)
{
	close_ncurses_screen();
	printf ("Your method which you sent to server is bad.\n"
		"Server can't understand this!, exiting...\n");

	exit(EXIT_FALSE);
}


void protocol_error(void)
{
	close_ncurses_screen();
	printf ("Server protocol is bad. "
		"This Client can't understand it!, exiting...\n");

	exit(EXIT_FALSE);
}


void server_name_error(void)
{
	printf ("Please enter correct server dns name\n"
		"or ip address with -s option, exiting...\n");

	exit(EXIT_FALSE);
}


void port_error(void)
{
	printf ("Server port "_STRING_WHITE_" is not valid,"
		" exiting...\n", optarg);

	exit(EXIT_FALSE);
}


void server_version_error(void)
{
	close_ncurses_screen();
	printf ("Server version "_STRING_WHITE_" too old, "
		"please use higher version, exiting...\n", 
		server_version);

	exit(EXIT_FALSE);
}


void message_error(void)
{
	printf ("Message which you want send to server is too long "
		"(max "_COLOR_WHITE_"%d"_COLOR_END_" chars.)\n", MESSAGE_MAX);

	exit(EXIT_FALSE);
}


void print_client_usage(void)
{
	printf ("Usage: %s [OPTIONS] ...", exec_name);
	printf ("\nExample: %s -s deth.org.pl -i eth1     "
		"<- Get data of interface eth1\n"
		"Options:\n"
		"  -s [NAME]	  --server        "
		"Choice server to connect\n"
		"  -i [INTERFACE]  --interface     "
		"Choice server interface\n"
		"  -p [PORT]       --port	  "
		"Choice server port (default: %d)\n"
		"  -d [DATA_TYPE]  --data	  "
		"Choice what we want get (MB<->KB<->B)\n"
		"  -m [MESSAGE]    --message	  "
		"Message which we want send to server\n"
		"  -l		  --license       Print license for this program\n"
		"  -v, -V  	  --version       Print program version\n"
		"  -h      	  --help          Print short help\n\n"
		"NOTE: If message is not one word then you must put it into "
		"quotation-mark!\n ->   %s -s deth.org.pl -p 667 -m \"This "
		"is message which you want to send\"\n\n", exec_name, 
		server_port, exec_name);

	exit(EXIT_SUCCESS);
}


/* NIN */

#endif /* _BCLIENT_H_ */

