/*
   *
   * bserver.c - Bytes Counter Server
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include "bserver.h"
#include "signals.h"
#include "get_request.h"
#include "bcnt_headers.h"
#include "http_headers.h"
#include "bcnt_bodys.h"
#include "http_bodys.h"
#include "bcnt_response.h"
#include "http_response.h"
#include "clear_buffer.h"
#include "license.h"



int main(int argc, char **argv, char **envp)
{
	int c;
	char* end;
	
	/* ! This is for bug report ! */
	strcpy (running_function, "main()");
	
	/* Signals */
	signals_functions();

	/* Set default terminal as "linux" -> TERM=linux */
	**envp = set_terminal_type(TERM_LINUX);

	exec_name = strrchr (argv[0], '/');
	if (!exec_name)
	exec_name = argv[0];
	else
	    ++exec_name;

	/* ****************************************************************** */
	/* IMPORTANT: This filling interface struct DON'T REMOVE THIS line! 
	    i->interfaces_names 
	    i->interface_counters 
	*/
	get_interfaces_in_system();

	/*  */
	if (search_for_interface() == INTERFACE_NOT_FOUND)
	    set_default_interface_as_loopback();
	
	/* Get default interface info ... like ip, netmask ...etc. */
	get_interface_data(default_interface_name);

	/* Get local host name */
	strcpy (local_host_name, get_local_host_name());
	strcpy (smtp_server_name, local_host_name);

	smtp_session_debug_flag = TRUE;
	/* ****************************************************************** */

	while ((c = getopt_long(argc, argv, "a:i:p:r:s:lgnm:vVh", 
				longopts, 0)) != EOF)
	    switch (c)
		{
		    case 'a':
			interface_bind_address = optarg;
			interface_bind_address_flag = TRUE;
			break;
		    case 'i':
			get_interface_data(optarg);
			break;
		    case 'p':
			server_port = atoi(optarg);
			if (server_port == 0)
			    port_error();
			break;
		    case 's':
			remote_smtp_server = optarg;
			break;
		    case 'r':
			xrefresh = atoi(optarg);
			strtol (optarg, &end, 10);
			if (*end != '\0')
			    refresh_error();
			break;
		    case 'n':
			resolve = 0;
			break;
		    case 'v':
		    case 'V':
			print_server_version();
			puts("");
			exit_prog(EXIT_SUCCESS);
			break;
		    case 'm':
			verbose_level = atoi(optarg);
			strtol (optarg, &end, 10);
			if (*end != '\0')
			    verbose_error();
			if (verbose_level != 1 && verbose_level != 2)
			    verbose_error();
			verbose++;
			break;
		    case 'l':
    			print_license();
			break;
		    case 'g':
    			signals_information();
			break;
		    case 'h':
			print_server_usage();
    			exit_prog(EXIT_SUCCESS);
			break;
		    default:
			print_info();
		}

	/* Interface threads starts */
	if (pthread_status == FALSE)
	    {
		/* Start thread interface stats read */
		pthread_stats_read_start();
		/* Start start interface speed measuring */
		pthread_speed_measure_start();
	    }

	/* Welcome to hell */
	print_welcome();

	/* Initializing SMTP Client */
	smtp_client_init();

	/* Run server */
	server_run();

	return(EXIT_SUCCESS);
}


void print_http_verbose1(void)
{
	printf ("\n "_COLOR_BLACK_"-------------- BEGIN_HTTP_HEADER_ANSWER "
		"--------------"_COLOR_END_"\n");
	printf (_STRING_RED_, header_buffer);
}


void print_http_verbose2(void)
{
	printf (" "_COLOR_BLACK_"--------------- BEGIN_HTTP_BODY_ANSWER "
		"---------------"_COLOR_END_"\n");
	printf (_STRING_BLUE_"\n", body_buffer);
}


void print_bcnt_verbose1(void)
{
	printf ("\n "_COLOR_BLACK_"-------------- BEGIN_BCNT_HEADER_ANSWER "
		"--------------"_COLOR_END_"\n");
	printf (_STRING_RED_, header_buffer);
}


void print_bcnt_verbose2(void)
{
	printf (" "_COLOR_BLACK_"--------------- BEGIN_BCNT_BODY_ANSWER "
		"---------------"_COLOR_END_"\n");
	printf (_STRING_BLUE_"\n", body_buffer);
}


void print_http_verbose(void)
{
	if (verbose == 1 && verbose_level == 1)
	    print_http_verbose1();
	else if (verbose == 1 && verbose_level == 2)
	    {
		print_http_verbose1();
		print_http_verbose2();
	    }
}


void print_bcnt_verbose(void)
{
	if (verbose == 1 && verbose_level == 1)
	    print_bcnt_verbose1();
	else if (verbose == 1 && verbose_level == 2)
	    {
		print_bcnt_verbose1();
		print_bcnt_verbose2();
	    }
}


void write_header_to_socket(void)
{
	write (client_socket, header_buffer, strlen(header_buffer));
}


void write_body_to_socket(void)
{
        write (client_socket, body_buffer, strlen(body_buffer));
}


void client_version_error(void)
{
	snprintf (body_buffer, sizeof(body_buffer), 
		LF BYTES_TX": 0"LF
		PACKETS_TX": 0"LF
		ERRORS_TX": 0"LF
		BYTES_RX": 0"LF
		PACKETS_RX": 0"LF
		ERRORS_RX": 0"LF
		MESSAGE": Client version too old, please upgrade this stuff!"LF
		END_BCNT_BODY LF);

        body_buffer[strlen (body_buffer)] = '\0';

	snprintf (header_buffer, sizeof(header_buffer), bcnt_ok_header, 
		     BCNT_PROTOCOL_DEVEL, "error", SNAME, 
		     get_local_host_name(), SVERSION, "error", strlen(body_buffer));

	header_buffer[strlen (header_buffer)] = '\0';

	write_header_to_socket();
	write_body_to_socket();
	print_http_verbose();

	exit_prog(EXIT_FALSE);
}


int handle_get (int client_socket, const char* what_want)
{
	char tmp_dev[15];
	char* fuck_client;
	char *what_want_ptr;
	int z = 0, n = 0;

	/* Checking Client version no matters if Bcount Client or WWW Client */
	if ((strstr (received_buffer, USER_AGENT)) != NULL)
	    {
		get_additional_request (received_buffer, USER_AGENT);

		if (!strcmp(protocol, BCNT_PROTOCOL_DEVEL) || 
		    !strcmp(protocol, BCNT_PROTOCOL_1_0))
		    {
			fuck_client = strstr(user_agent, "Version");

			if (!strncmp (fuck_client, "Version 0.6", 12))
			    client_version_error();
			else if (!strncmp (fuck_client, "Version 0.5", 11))
			    client_version_error();
			else if (!strncmp (fuck_client, "Version 0.4", 11))
			    client_version_error();
		    }

	    }
	else
	    strcpy (user_agent, "Unknown! Who is that Client?!");

	syslog (LOG_ERR, "Cv=%s", user_agent);

	/* Checking others Client requests ... */
	if ((strstr (received_buffer, DATA_TYPE) != NULL))
	    get_additional_request (received_buffer, DATA_TYPE);

	printf (" Clt :: "_STRING_YELLOW_" ::\n"
		" Prl :: "_STRING_GREEN_" ::\n", 
		user_agent, protocol); 

	if (!strcmp (what_want, "/"))
            printf (" Get :: "_COLOR_BLUE_"%s%s (default interface)"
		    _COLOR_END_" ::\n", what_want, i->name);
	else
	    printf (" Get :: "_STRING_BLUE_" ::\n", what_want);

	if ((strstr (received_buffer, MESSAGE) != NULL))
	    {
		get_additional_request (received_buffer, MESSAGE);

		printf (" Msg :: "_STRING_CYAN_" ::\n", cmessage);
		syslog (LOG_ERR, "message=%s", cmessage);
	    }

	/* Check if only one slash '/'. If not then message "Not Found"*/
	if (!strcmp (what_want, "/"))
	    { 
		if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
		    !strcmp (protocol, HTTP_PROTOCOL_1_1))
		    {
			http_interface_data_response(i->name);

			write_header_to_socket();
			write_body_to_socket();
			print_http_verbose();
		    }
		else if (!strcmp (protocol, BCNT_PROTOCOL_1_0) || 
			 !strcmp (protocol, BCNT_PROTOCOL_DEVEL))
		    {
			bcnt_interface_data_response(i->name);

			write_header_to_socket();
			write_body_to_socket();
			print_bcnt_verbose();
		    }
	    } 
	else if ((strstr (what_want, "?refresh=") != NULL))
	    { 
		what_want_ptr = strstr (what_want, "?refresh=");

		printf (" -> Client changed refresh time from "
			_COLOR_WHITE_"%d"_COLOR_END_, xrefresh);

		xrefresh = atoi(what_want_ptr + (strlen("?refresh=")));

		printf (" to "_COLOR_WHITE_"%d"_COLOR_END_"!\n", xrefresh);

		if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
		    !strcmp (protocol, HTTP_PROTOCOL_1_1))
		    {
			http_interface_data_response(i->name);

			write_header_to_socket();
			write_body_to_socket();
			print_http_verbose();
		    }

		return(EXIT_SUCCESS);
	    } 
	else if ((strstr (what_want, "?interface=") != NULL))
	    { 
		what_want_ptr = strstr (what_want, "?interface=");
		strcpy (tmp_dev, i->name);
		strcpy (i->name, (what_want_ptr + (strlen("?interface="))));

		if (search_for_interface(i->name) != INTERFACE_NOT_FOUND)
		    {
			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
				get_interface_data(i->name);
				http_interface_data_response(i->name);

				write_header_to_socket();
				write_body_to_socket();
				print_http_verbose();
			    }
		    }
		else
		    {
			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
				http_interface_not_found_response(i->name); 

				write_header_to_socket();
				write_body_to_socket();
				print_http_verbose();
			    }

			strcpy (i->name, tmp_dev);
			get_interface_data(i->name);
		    }

		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		return(EXIT_SUCCESS);
	    }
	else if ((strstr (what_want, "/interface-info") != NULL))
	    { 
		z = 1;
		n = 0;

		while (what_want[z] != '/')
		    {
			tmp_dev[n] = what_want[z];
			z++;
			n++;
		    } 

		tmp_dev[n] = '\0';

		if (search_for_interface(tmp_dev) != INTERFACE_NOT_FOUND)
		    {
			strcpy (i->name, tmp_dev);

			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
				get_interface_data(i->name);
				http_interface_info_response(i->name);

				write_header_to_socket();
				write_body_to_socket();
				print_http_verbose();
			    }
		    }
		else
		    {
			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
				http_interface_not_found_response(tmp_dev); 

				write_header_to_socket();
				write_body_to_socket();
				print_http_verbose();
			    }
		    }

		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		return(EXIT_SUCCESS);
	    }
	else if (!strcmp (what_want, "/penguin.gif"))
	    { 
		if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
		    !strcmp (protocol, HTTP_PROTOCOL_1_1))
		    {
			write_gif_file_to_stream("penguin.gif", client_socket);
			print_http_verbose();
		    }

		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		return(EXIT_SUCCESS);
	    } 
	else if (!strcmp (what_want, "/server-info"))
	    { 
		if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
		    !strcmp (protocol, HTTP_PROTOCOL_1_1))
		    {
			http_server_info_response();

			write_header_to_socket();
			write_body_to_socket();
			print_http_verbose();
		    }

		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		return(EXIT_SUCCESS);
	    } 
	else if (!strcmp (what_want, "/setup"))
	    { 
		if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
		    !strcmp (protocol, HTTP_PROTOCOL_1_1))
		    {
			http_server_info_response();

			write_header_to_socket();
			write_body_to_socket();
			print_http_verbose();
		    }

		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		return(EXIT_SUCCESS);
	    } 
	else if (!strcmp (what_want, "/about"))
	    { 
		if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
		    !strcmp (protocol, HTTP_PROTOCOL_1_1))
		    {
			http_about_response();

			write_header_to_socket();
			write_body_to_socket();
			print_http_verbose();
		    }

		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		return(EXIT_SUCCESS);
	    } 
	else
	    {
		z = 1;
		n = 0;

		while (what_want[z] != '\0')
		    {
			tmp_dev[n] = what_want[z];
			z++;
			n++;
		    } 

		tmp_dev[n] = '\0';

		if (search_for_interface(tmp_dev) != INTERFACE_NOT_FOUND)
		    {
			strcpy (i->name, tmp_dev);

			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
                                http_interface_data_response(i->name);

				write_header_to_socket();
				write_body_to_socket();
				print_http_verbose();
			    }
			else if (!strcmp (protocol, BCNT_PROTOCOL_DEVEL) || 
				 !strcmp (protocol, BCNT_PROTOCOL_1_0))
			    {
				bcnt_interface_data_response(i->name);

				write_header_to_socket();
				write_body_to_socket();
				print_bcnt_verbose();
			    }

    		    }
		else
		    {
			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
				http_interface_not_found_response(tmp_dev); 

				write_header_to_socket();
				write_body_to_socket();
				print_http_verbose();
			    }
			else if (!strcmp (protocol, BCNT_PROTOCOL_DEVEL) || 
			    !strcmp (protocol, BCNT_PROTOCOL_1_0))
			    {
				bcnt_interface_not_found_response(tmp_dev); 

				write_header_to_socket();
				print_bcnt_verbose();
			    }
		    }
	    }

	clear_buffer(header_buffer);
	clear_buffer(body_buffer);

        return(EXIT_SUCCESS);
}


int handle_connection (int client_socket)
{
	ssize_t bytes_read = 0;
	char url[50];

	clear_buffer(received_buffer);
	
	/* Reading data from Client until `\r\n\r\n' is not detected */
	while (strstr (received_buffer, CRLF CRLF) == NULL) 
	/* Read data received from client */
	    bytes_read = read (client_socket, received_buffer, 
				sizeof(received_buffer) - 1);
	if (bytes_read < 0)
	    {
		fprintf(stderr, _STRING_RED_": handle_connection() "
			"-> read()\n", strerror(errno));
		closelog();

		return(EXIT_FALSE);
	    }

	if (bytes_read > 0)
	    {
		/* On the end of received message we put NULL byte */
		received_buffer[bytes_read] = '\0';

		/* Reading Client protocol, method and whats Client want */
		sscanf (received_buffer, "%s %s %s", method, url, protocol);

		/* Send message to old client then exit */
		if (!strcmp (method, "0.1") || !strcmp (method, "0.2"))
		    {
			snprintf (body_buffer, sizeof(body_buffer), 
				    _COLOR_RED_"This client is very old ;-(. "
				    "Please upgrade this program!"_COLOR_END_
				    "          \n\n\n\n");
			printf (" Clt :: "_STRING_YELLOW_DARK_" :: "
				"...This Client is still alife?!  He must die"
				" :-)\n", method);

			write (client_socket, body_buffer, 
				strlen(body_buffer));

			close (client_socket);

			exit_prog(EXIT_FALSE);
		    }

		/* Send message to old Windows client then exit */
		if (!strcmp (method, "0.1win"))
		    {
			snprintf (body_buffer, sizeof(body_buffer), 
				    CRLF"This client is very old ;-(."CRLF
				    "Please upgrade this program!"CRLF);

			printf (" Clt :: "_STRING_YELLOW_DARK_" :: "
				"...This Client is still alife?!  He must die"
				" :-)"LF, method);

			write (client_socket, body_buffer, 
				strlen(body_buffer));

			close (client_socket);

			exit_prog(EXIT_FALSE);
		    }

		/* Yes we support HTTP protocol ver. 1.0/1.1 
		    (not all impl.) ... */   
		if ((strcmp (protocol, HTTP_PROTOCOL_1_0) && 
		    strcmp (protocol, HTTP_PROTOCOL_1_1)) &&

		/* ... and support BCOUNT protocol ver. 0.9 
		    and ver. 1.0 (for future) */
		    (strcmp (protocol, BCNT_PROTOCOL_DEVEL) && 
		    strcmp (protocol, BCNT_PROTOCOL_1_0)))
		    {
			http_bad_request_response(protocol); 

			write (client_socket, header_buffer, 
				strlen(header_buffer));

			write (client_socket, body_buffer, 
				strlen(body_buffer));

			printf(" Prl :: "_STRING_RED_" :: "
				"...Client sent Bad Request!\n\n", protocol);
			if (verbose == 1 && verbose_level == 1)
			    print_http_verbose1();
			else if (verbose == 1 && verbose_level == 2)
			    {
				print_http_verbose1();
				print_http_verbose2();
			    }

			close (client_socket);

			exit_prog(EXIT_FALSE);
		    }
		/* We only support GET method */
		else if (strcmp (method, GET_METHOD))
		    {
			printf (" Mtd :: "_STRING_RED_" :: "
				"...This Client sent a "
    	    			"request that this server could not "
	    			"understand!\n\n", method);

			if (!strcmp (protocol, HTTP_PROTOCOL_1_0) || 
			    !strcmp (protocol, HTTP_PROTOCOL_1_1))
			    {
				http_bad_method_response(method);

				write (client_socket, header_buffer, 
					strlen(header_buffer));

				write (client_socket, body_buffer, 
					strlen(body_buffer));

				if (verbose == 1 && verbose_level == 1)
				    print_http_verbose1();
				else if (verbose == 1 && verbose_level == 2)
				    {
					print_http_verbose1();
					print_http_verbose2();
				    }
			    }
			if (!strcmp (protocol, BCNT_PROTOCOL_DEVEL) || 
			    !strcmp (protocol, BCNT_PROTOCOL_1_0))
			    {
				bcnt_bad_method_response(method);

				write (client_socket, header_buffer, 
					strlen(header_buffer));

				if (verbose == 1 && verbose_level == 1)
				    print_bcnt_verbose1();
				else if (verbose == 1 && verbose_level == 2)
				    {
					print_bcnt_verbose1();
					print_bcnt_verbose2();
				    }
			    }

			close (client_socket);

			exit_prog(EXIT_FALSE);
		    }    
		else
		    /* Ok now we touching data for send to Client */
		    handle_get (client_socket, url);
	    }
	/* Do nothing if Client don't asking */
	else if (bytes_read == 0)
	    ;
	else
	    fprintf(stderr, _STRING_RED_"\n", strerror(errno));
    	    
	return(EXIT_SUCCESS);
}


/* Standalone Server for foreign connections */
int server_run(void)
{
	struct sockaddr_in server_address;
	struct sigaction sigchld_action;
	int so_reuseaddr = 1;

	/* Yes ...All the Children will be dying ;-) */
	memset (&sigchld_action, 0, sizeof(sigchld_action));
	sigchld_action.sa_handler = &clean_up_child_process;
	sigaction (SIGCHLD, &sigchld_action, NULL);

	/* Open TCP stream socket */
	server_socket = inet_socket_stream();

	/* SO_REUSEADDR help if we want run several times `bserver' then
	    bind() function can't exit with "Address is alredy in use" :-) */
	if ((setsockopt (server_socket, SOL_SOCKET, SO_REUSEADDR, 
			(char*)&so_reuseaddr, sizeof(so_reuseaddr))) < 0)
	    {
		fprintf(stderr, _STRING_RED_": server_run() -> "
		"setsockopt()\n", strerror(errno));
	    }

	/* Internet address for family addresses */
	server_address.sin_family = PF_INET;

	/* Host order (little-endian or big-endian) 
	    to network order (always big-endian) for Port */
	server_address.sin_port = htons (server_port);

	if (interface_bind_address_flag == TRUE)
	    {
		/* Bind socket with address */
		server_address.sin_addr.s_addr = inet_addr(interface_bind_address);
		if (server_address.sin_addr.s_addr == INADDR_NONE)
		    interface_address_error();		
	    }
	else
	    /* Bind socket with address any interface */
	    server_address.sin_addr.s_addr = INADDR_ANY;

	/* Reseting `server_address.sin_zero' */
	bzero(&(server_address.sin_zero), 8);


	if (bind (server_socket, (struct sockaddr *) &server_address, 
		    sizeof (server_address)))
	    {
		if (errno == EACCES)
		    fprintf(stderr, _STRING_RED_": server_run() -> bind()\n"
			    "Remember than only Root can use ports from"
			    " 1 to 1023! If you a normal ;-) user\n"
			    "please run server on port biger than 1023! "
			    "then this stuff will be working :-)\n", 
			    strerror(errno));

		if (errno == EADDRINUSE)
		    fprintf(stderr, _STRING_RED_": server_run() -> bind()\n"
			    "Please check if other server session not "
			    "running on port: "_COLOR_WHITE_"%d"_COLOR_END_
			    " :-)\n", strerror(errno), server_port);

		exit_prog(EXIT_FALSE);
	    }

	if (listen (server_socket, CLIENT_QUEUE))
	    {
		fprintf(stderr, _STRING_RED_": server_run() -> "
			"listen()\n", strerror(errno));

		exit_prog(EXIT_FALSE);
	    }

	printf (":: Examining default interface ::\n"
	        "-> Interface name: %s [%s]\n", i->name, i->description);

	hex_address = inet_network(i->ip_addr);
	printf ("   Address IP: %s 0x%08lx(BE)\n", i->ip_addr, 
		(unsigned long)htonl(hex_address));

	if (i->ltype == (int *) LTYPE_PPP || i->ltype == (int *) LTYPE_IPPP)
	    {
		hex_address = inet_network(i->ip_daddr);
		printf ("   Address PPP IP: %s 0x%08lx(BE)\n", i->ip_daddr, 
			(unsigned long)htonl(hex_address));
	    }

	hex_address = inet_network(i->netmask);
	printf ("   Netmask: %s 0x%08lx(BE)\n", i->netmask, 
		(unsigned long)htonl(hex_address));

	if (strcmp (i->broadcast, NOT_AVAILABLE))
	    {
		hex_address = inet_network(i->broadcast);
		printf ("   Broadcast: %s 0x%08lx(BE)\n", i->broadcast, 
			(unsigned long)htonl(hex_address));
	    }

	if (strcmp (i->hw_addr, NOT_AVAILABLE))
	    printf ("   HW address: %s (MAC)\n", i->hw_addr);

	if (interface_bind_address_flag == TRUE)
	    printf ("-> Bind with address: %s\n", 
		    interface_bind_address);
	else
	    printf ("-> Bind with `any' address on this host\n");

	printf ("-> Listening on port: %d\n\n"
		"Waiting for clients...\n"
		"[Press Ctrl+C to exit]\n",
		server_port);

	/* Open log file */
	openlog (exec_name, LOG_PID, LOG_DAEMON);
	syslog (LOG_ERR, "%s is running... "
		"Listening on port:%d Sv=%s", 
		SNAME, server_port, SVERSION);

	/* Neverending story */
        while(1)
            {
                struct sockaddr_in client_address;
    		struct hostent *client_hostent;
                socklen_t address_lenght;
                pid_t child_pid;

                address_lenght = sizeof(client_address);
                client_socket = accept (server_socket, (struct sockaddr *)
					&client_address, &address_lenght);
                if (client_socket < 0)
                    {
			/* If EINTR get signal then continue */
			if (errno == EINTR)
			    continue;
			else
			    {
				fprintf(stderr, _STRING_RED_
					": Lost a connection -> accept()\n", 
					hstrerror(h_errno));
			    }
		    }

		snprintf (body_buffer, sizeof(body_buffer), 
		    "+"STRING_OK" %s [%s] %s %s v%s ready "HAPPY_FACE"\n", 
		    get_local_host_name(), i->ip_addr, 
		    get_local_host_info(HOST_ARCHITECTURE), 
		    SNAME, SVERSION);

		write (client_socket, body_buffer, strlen(body_buffer));
		
		if (resolve)
		    {
			/* Get client host name from IP */
			client_hostent = gethostbyaddr (
					(char *) &client_address.sin_addr, 
					sizeof(client_address.sin_addr),
					client_address.sin_family);

			if (client_hostent == NULL)
			    resolve_ok = 0;
			else
			    resolve_ok = 1;

		    }
		else
		    /* To shut gcc up! */
		    client_hostent = NULL;
		    
		/* Increase connection counter */
		how_many_connections++;

		/* Clear buffer when children will born */
		clear_buffer(header_buffer);
		clear_buffer(body_buffer);

		/* Here is Parent heart ...or something else :-) */
		child_pid = fork();
		if (child_pid == 0)
		    { 
			close (server_socket);

			printf ("\n"_COLOR_WHITE_"[%s %s]"_COLOR_END_"\n",
				get_localdate(), get_localtime());

			hex_address = inet_network(inet_ntoa (client_address.sin_addr));

			printf (" Hst :: "_COLOR_RED_"%s:%i "
				"0x%08lx(BE)"_COLOR_END_" ",	
				inet_ntoa (client_address.sin_addr), 
				ntohs (client_address.sin_port),
				(unsigned long)htonl(hex_address));

			if (resolve && resolve_ok)
			    {
				printf (_COLOR_RED_"[%s]"
					_COLOR_END_" ::\n", 
					client_hostent->h_name);

				syslog (LOG_ERR, "From %s:%i [%s]", 
					inet_ntoa (client_address.sin_addr), 
					ntohs (client_address.sin_port), 
					client_hostent->h_name);
			    }
			else
			    {
				printf ("::\n");

				syslog (LOG_ERR, "From %s:%i", 
					inet_ntoa (client_address.sin_addr), 
					ntohs (client_address.sin_port)); 
			    }

			/* Serve to client connection */
			handle_connection (client_socket);

			close(client_socket);

			exit_prog(EXIT_SUCCESS);
		    } 

		/* If we are parents... */
		else if (child_pid > 0)
		    close(client_socket);
		else
		    {
			if (errno == EAGAIN)
			    {
				fprintf(stderr, _STRING_RED_": "
					"server_run() -> fork()\nOoh, "
					"what a bitch kill my server! Too "
					"much Clients want data from me, "
					"Fuck Off!\n", strerror(h_errno));
			    }
			else
			    {
				fprintf(stderr, _STRING_RED_": "
					"server_run() -> fork()\n", 
					strerror(h_errno));
			    }
			
			closelog();

			close(client_socket);
			close(server_socket);
			
			exit_prog(EXIT_FALSE);
		    }

	}

}


/* NIN */

